#!/usr/bin/env python

import zlib
import subprocess as sp

def deflate(filename, outfile=None, level=6):
	f = open(filename)
	data = f.read()
	f.close()

	compress = zlib.compressobj(
		level,                # level: 0-9
		zlib.DEFLATED,        # method: must be DEFLATED
		-zlib.MAX_WBITS,      # window size in bits:
		                      #   -15..-8: negate, suppress header
		                      #   8..15: normal
		                      #   16..30: subtract 16, gzip header
		zlib.DEF_MEM_LEVEL,   # mem level: 1..8/9
		0                     # strategy:
		                      #   0 = Z_DEFAULT_STRATEGY
		                      #   1 = Z_FILTERED
		                      #   2 = Z_HUFFMAN_ONLY
		                      #   3 = Z_RLE
		                      #   4 = Z_FIXED
	)
	deflated = compress.compress(data)
	deflated += compress.flush()

	if outfile != None:
		f = open(outfile, 'w')
		f.write(deflated)
		f.close()

	return deflated

def inflate(filename, outfile=None):
	f = open(filename)
	data = f.read()
	f.close()

	decompress = zlib.decompressobj(-zlib.MAX_WBITS)  # see above
	inflated = decompress.decompress(data)
	inflated += decompress.flush()

	if outfile != None:
		f = open(outfile, 'w')
		f.write(inflated)
		f.close()

	return inflated

"""
Convenience function for running a command bash-like

@param command- string version of a command to run on
@param shell- Whether to run this through the shell; used in subprocess.Popen (default: true)
@return Object with properties 'returncode', 'stdout', and 'stderr'
"""
def run_cmd(command, shell=True):
	process = sp.Popen(command, shell=shell, stdout = sp.PIPE, stderr = sp.PIPE)
	stdout, stderr = process.communicate()
	returncode = process.returncode
	return {'returncode' : returncode, 'stdout' : stdout, 'stderr' : stderr}



#!/usr/bin/env python

import argparse
import deflate
import inflate
from colorama import Fore

testDir = 'test-files'
outDir = 'test-outs'

allPassed = True

parser = argparse.ArgumentParser(description='Process command-line arguments')
parser.add_argument('--test', metavar='path/to/file', type=str, default='both', nargs='?', help='Which test to run: deflate, inflate, or both')
parser.add_argument('--file', '-f', metavar='path/to/file', type=str, nargs='?', help='Path to file to use for test')
parser.add_argument('--level', '-l', metavar='#', type=int, nargs='?', help='Compression level')
parser.add_argument('--no-delete', const=True, default=False, nargs='?', help='Don\'t delete files produced for test')

args = parser.parse_args()

delete = not getattr(args, 'no_delete')
level = getattr(args, 'level')
inFile = getattr(args, 'file')
test = getattr(args, 'test')

if test == 'deflate' or test == 'both':
	print Fore.CYAN + 'Running deflate tests' + Fore.RESET

	passed = True

	if inFile != None:
		passed = deflate.runTest(inFile, level, delete, outDir)
	else:
		passed = deflate.runAll(level, delete, testDir, outDir)

	# if we fail one test, we fail the entire test	
	allPassed = allPassed and passed

if test == 'inflate' or test == 'both':
	print Fore.CYAN + 'Running inflate tests' + Fore.RESET

	passed = True

	if inFile != None:
		passed = inflate.runTest(inFile, level, delete, outDir)
	else:
		passed = inflate.runAll(level, delete, testDir, outDir)
	
	# if we fail one test, we fail the entire test	
	allPassed = allPassed and passed

if allPassed:
	print Fore.GREEN + 'All tests passed!' + Fore.RESET
else:
	print Fore.RED + 'Automated test failed' + Fore.RESET



import os
from colorama import Fore
from helpers import deflate, inflate, run_cmd

outDirDefault = 'test-outs'
testDirDefault = 'test-files'

"""
Run a single test

@param tFile- required; the file to check against (uncompressed data)
@param level- optional (default: all); the compression level [1-9]
@param delete- optional (default: True); whether to delete the gzipped files
@return True if all tests passed; False if at least one test failed
"""
def runTest(tFile, level=None, delete=True, outDir=outDirDefault):
	passed = True

	if level == None:
		for x in range(1, 10):
			if runTest(tFile, x, delete) == False:
				passed = False

		return passed

	# make the test-outs directory
	try:
		os.mkdir(outDir)
	except:
		pass

	out1 = os.path.join(outDir, '%(file)s.%(level)d.deflate' % {'file': os.path.basename(tFile), 'level' : level})
	out2 = os.path.join(outDir, '%(file)s.%(level)d' % {'file' : os.path.basename(tFile), 'level' : level})

	deflate(tFile, outfile=out1, level=level)
	run_cmd('../bin/inflate.js --file %(file)s --output %(output)s' % {'level' : level, 'file' : out1, 'output' : out2})

	result = run_cmd('diff %(file1)s %(file2)s' % {'file1' : tFile, 'file2' : out2})
	if result['returncode'] == 0:
		status = Fore.GREEN + 'PASSED' + Fore.RESET
	else:
		passed = False
		status = Fore.RED + 'FAILED' + Fore.RESET
	
	print 'Level %(level)d: %(status)s' % {'level' : level, 'status' : status}

	if delete == True:
		os.remove(out1)
		os.remove(out2)

	return passed

"""
Runs all tests on the given level. This iterates throuth the testDir directory defined above.

@param level- The level to run on [1-9] (default: None, runs on all levels all)
@param delete- Whether to delete output files after the test is run
@return True if all levels passed, False if at least one failed
"""
def runAll(level=None, delete=True, testDir=testDirDefault, outDir=outDirDefault):
	# make the test-outs directory
	try:
		os.mkdir(outDir)
	except:
		pass

	passed = True
	for tFile in os.listdir(testDir):
		fullPath = os.path.join(testDir, tFile)

		print Fore.YELLOW + tFile + Fore.RESET

		if runTest(fullPath, level, delete) == False:
			passed = False

		print ''
	
	# if we deletede all the files that were created, delete the directory
	if delete == True:
		os.rmdir(outDir)
		return passed
